==> notftp-1.3.tbz: www/data/notftp/functions.php <==
<?
/********************************************************************
 * NotFTP 
 * Copyright (c) 2002 by Ryan Grove <ryan@wonko.com>
 * All rights reserved.
 *
 * This script is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA  02111-1307  USA
 ********************************************************************/

require_once("config.php");

$debug = false; // Set to true to display debugging information

/**
  * Searches for a regex match in every value of a one-dimensional array.
  *
  * @param string $pattern POSIX-extended regular expression to match.
  * @param array $haystack Array to search in.
  * @return mixed Index of the first match, or FALSE if no match is found.
  */
function array_ereg($pattern, $haystack)
{
	for($i = 0; $i < count($haystack); $i++)
	{
		if (ereg($pattern, $haystack[$i]))
			return $i;
	}
	
	return false;
}

/**
  * Attempts to determine the MIME type of a file based on its extension.
  *
  * Attempts to determine the MIME type of a file based on its extension,
  * using the standard MIME type registry from the IANA. If the MIME type
  * can't be determined, defaults to "text/plain".
  * 
  * @param string $file Filename
  * @return string MIME type
  */
function getMIMEType($file)
{
    global $mimetypes;

    // Get the extension
    $pos = strrpos($file, ".");

    if ($pos !== false)
        $ext = substr($file, $pos + 1);
    else
        return "text/plain";

    // Find this extension in our MIME type array
    foreach($mimetypes as $mime)
    {
        if (eregi($ext, $mime))
        {
            prev($mimetypes);
            return key($mimetypes);
        }
    }

    // No MIME type found, return text/plain
    return "text/plain";
}

/**
  * Converts the output of ftp_rawlist() to an array that
  * is structured similarly to the output of ftp_nlist().
  *
  * @param array $list Array returned by ftp_rawlist()
  * @return array
  */
function rawlist2nlist($list)
{
    $newlist = array();
    @reset($list);

    while (list(,$row) = @each($list))
    {
        $index = strrpos($row, " ");
        $newlist[] = substr($row, $index+1);
    }

    return $newlist;
}

/**
  * Given an entry from the output of ftp_rawlist(),
  * attempts to determine whether the entry is a
  * directory. 
  *
  * @param string $item Entry from ftp_rawlist()
  * @return bool TRUE for a directory, otherwise FALSE
  */
function isdir($item)
{
    global $systype;

    // some rawlists returned by ftp_rawlist() contain '.' and '..' directory
    // so we ignore them, even they have the 'd' permisson on
    if (eregi("windows", $systype))
    {
        if (ereg("^d[rtwx\-]{9}", $item))
            return true;
        else
            return false;
    }
    else
    {
        if (ereg("^d[rstwx\-]{9}", $item))
            return true;
        else
            return false;
    }
}

/**
  * Given an entry from the output of ftp_rawlist(),
  * attempts to determine whether the entry is a
  * file.
  *
  * @param string $item Entry from ftp_rawlist()
  * @return bool TRUE for a file, otherwise FALSE
  */

function isfile($item)
{
    if (ereg("^[rtwxs\-]{10}", $item))
        return true;
    else
        return false;
}

/**
  * Given an entry from the output of ftp_rawlist(), attempts to determine
  * whether or not the entry is a symbolic link, and whether or not it's
  * a link to a file or a directory. This function isn't complete yet.
  *
  * @param string $item Entry from ftp_rawlist()
  * @param array $rawlist The actual full output of ftp_rawlist()
  * @return integer 1 for a file, 2 for a directory, or 0 on error or if the entry is not a symlink
  */
function islink($item, $rawlist)
{
	if (ereg("^l[rtwxs\-]{9}", $item))
	{
		// It's a symlink, but to what?
		$matches = array();
		preg_match("/.+ -> (.+)/", $item, $matches);
		
		$target = $matches[1];
		// Is target in relation to the current directory?
		if (!ereg("^/", $target))
		{
			// Search for target in rawlist
			if ($index = array_ereg($target, $rawlist))
			{
			print "YAY";
				if (isfile($rawlist[$index]))
					return 1;
				elseif (isdir($rawlist[$index]))
					return 2;
				else
					return 0;
			}
			else
			{
				// Couldn't find the target
				return 0;
			}
		}			
	}
	else
	{
		// No symlink here
		return 0;
	}
}

/**
  * Given an entry from the output of ftp_rawlist(),
  * attempts to return the date of the file or directory.
  *
  * @param string $item Entry from ftp_rawlist()
  * @return string File date
  */
function getFileDate($item)
{
    global $conn;

	$matches = array();
	eregi("([a-z]{3} +[0-9]{1,2} [ 0-9\:]{4,5})", $item, $matches);

	return $matches[1];
}

/**
  * Given an entry from the output of ftp_rawlist(),
  * attempts to return the name of the file or
  * directory.
  *
  * @param string $item Entry from ftp_rawlist()
  * @return string Name of file or directory
  */
function getName($item)
{
    $matches = array();
    eregi("[a-z]{3} +[0-9]{1,2} [ 0-9\:]{4,5} (.+)$", $item, $matches);

    return $matches[1];
}

/**
  * Given an entry from the output of ftp_rawlist(),
  * attempts to return the permissions of the item.
  *
  * @param string $item Entry from ftp_rawlist()
  * @return string Permissions
  */
function getPermissions($item)
{
    $matches = array();

    eregi("^(.{10}) ", $item, $matches);
    return $matches[1];
}

/**
  * Given a file entry from the output of ftp_rawlist(),
  * attempts to return the size of the file in bytes.
  *
  * @param string $item Entry from ftp_rawlist()
  * @return mixed Size in bytes
  */
function getSize($item)
{
    $matches = array();
    eregi("^.{10}[ ]+[0-9]+ [^ ]+ +[^ ]+ +([0-9]+) ", $item, $matches);

    return $matches[1];
}

/**
  * Given a file entry from the output of ftp_rawlist(),
  * attempts to return the owner of the file
  *
  * @param string $item Entry from ftp_rawlist()
  * @return string Owner
  */
function getOwner($item)
{
    $matches = array();
    eregi("^.{10}[ ]+[0-9]+ ([^ ]+) ", $item, $matches);

    return $matches[1];
}

/**
  * Given a file entry from the output of ftp_rawlist(),
  * attempts to return the group of the file
  *
  * @param string $item Entry from ftp_rawlist()
  * @return string Group name
  */
function getGroup($item)
{
    $matches = array();
    eregi("^.{10}[ ]+[0-9]+ [^ ]+ +([^ ]+) ", $item, $matches);

    return $matches[1];
}

/**
  * Attempts to delete a directory and its contents recursively. (This
  * doesn't work yet.)
  * 
  * @param string $dir Directory to delete
  * @return bool TRUE on success, FALSE on failure.
  */
function recurseDelete($dir)
{
    global $systype, $conn;

    $rawlist=ftp_rawlist($conn, $dir);

    foreach ($rawlist as $item)
    {
        // If it's a file, delete it
        if (isfile($item))
        {
            if (!@ftp_delete($conn, $dir."/".getName($item)))
            {
                return false;
            } // Can't delete
        }

        // If it's a directory, then recurse
        if (isdir($item))
        {
            if (!(recurseDelete($dir."/".getName($item))))
            {
                return false;
            } // Can't delete
        }
    }

    // Now remove the directory itself
    if (!@ftp_rmdir($conn, $dir))
    {
        return false;
    } // Can't remove

    return true;
}
?>

==> notftp-1.3.1.tbz: www/data/notftp/functions.php <==
<?
/********************************************************************
 * NotFTP 
 * Copyright (c) 2002 by Ryan Grove <ryan@wonko.com>
 * All rights reserved.
 *
 * This script is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA  02111-1307  USA
 ********************************************************************/

require_once("config.php");

$debug = false; // Set to true to display debugging information

/**
  * Searches for a regex match in every value of a one-dimensional array.
  *
  * @param string $pattern POSIX-extended regular expression to match.
  * @param array $haystack Array to search in.
  * @return mixed Index of the first match, or FALSE if no match is found.
  */
function array_ereg($pattern, $haystack)
{
	for($i = 0; $i < count($haystack); $i++)
	{
		if (ereg($pattern, $haystack[$i]))
			return $i;
	}
	
	return false;
}

/**
  * Attempts to determine the MIME type of a file based on its extension.
  *
  * Attempts to determine the MIME type of a file based on its extension,
  * using the standard MIME type registry from the IANA. If the MIME type
  * can't be determined, defaults to "text/plain".
  * 
  * @param string $file Filename
  * @return string MIME type
  */
function getMIMEType($file)
{
    global $mimetypes;

    // Get the extension
    $pos = strrpos($file, ".");

    if ($pos !== false)
        $ext = substr($file, $pos + 1);
    else
        return "text/plain";

    // Find this extension in our MIME type array
    foreach($mimetypes as $mime)
    {
        if (eregi($ext, $mime))
        {
            prev($mimetypes);
            return key($mimetypes);
        }
    }

    // No MIME type found, return text/plain
    return "text/plain";
}

/**
  * Converts the output of ftp_rawlist() to an array that
  * is structured similarly to the output of ftp_nlist().
  *
  * @param array $list Array returned by ftp_rawlist()
  * @return array
  */
function rawlist2nlist($list)
{
    $newlist = array();
    @reset($list);

    while (list(,$row) = @each($list))
    {
        $index = strrpos($row, " ");
        $newlist[] = substr($row, $index+1);
    }

    return $newlist;
}

/**
  * Given an entry from the output of ftp_rawlist(),
  * attempts to determine whether the entry is a
  * directory. 
  *
  * @param string $item Entry from ftp_rawlist()
  * @return bool TRUE for a directory, otherwise FALSE
  */
function isdir($item)
{
    global $systype;

    // some rawlists returned by ftp_rawlist() contain '.' and '..' directory
    // so we ignore them, even they have the 'd' permisson on
    if (eregi("windows", $systype))
    {
        if (ereg("^d[rtwx\-]{9}", $item))
            return true;
        else
            return false;
    }
    else
    {
        if (ereg("^d[rstwx\-]{9}", $item))
            return true;
        else
            return false;
    }
}

/**
  * Given an entry from the output of ftp_rawlist(),
  * attempts to determine whether the entry is a
  * file.
  *
  * @param string $item Entry from ftp_rawlist()
  * @return bool TRUE for a file, otherwise FALSE
  */

function isfile($item)
{
    if (ereg("^[rtwxs\-]{10}", $item))
        return true;
    else
        return false;
}

/**
  * Given an entry from the output of ftp_rawlist(), attempts to determine
  * whether or not the entry is a symbolic link, and whether or not it's
  * a link to a file or a directory. This function isn't complete yet.
  *
  * @param conn FTP connection resource.
  * @param item Entry from ftp_rawlist()
  * @return 1 for a file, 2 for a directory, or false on error or if the entry is not a symlink.
  */
function islink($conn, $item)
{
	if (ereg("^l[rtwxs\-]{9}", $item))
	{
		// It's a symlink, but to what?
		$matches = array();
		preg_match("/.+ -> (.+)/", $item, $matches);
		
		$target = $matches[1];
		
		if (@ftp_chdir($conn, $target))
			return 2;
		else
			return 1;
	}
	else
	{
		// No symlink here.
		return false;
	}
}

/**
  * Given an entry from the output of ftp_rawlist(),
  * attempts to return the date of the file or directory.
  *
  * @param string $item Entry from ftp_rawlist()
  * @return string File date
  */
function getFileDate($item)
{
    global $conn;

	$matches = array();
	eregi("([a-z]{3} +[0-9]{1,2} [ 0-9\:]{4,5})", $item, $matches);

	return $matches[1];
}

/**
  * Given an entry from the output of ftp_rawlist(), attempts to return the
  * name of the file, directory or symlink.
  *
  * @param item Entry from ftp_rawlist().
  * @return Name of file, directory or symlink.
  */
function getName($item)
{
    $matches = array();
    eregi("[a-z]{3} +[0-9]{1,2} [ 0-9\:]{4,5} ([^( \\-\\>)]+)", $item, $matches);

    return $matches[1];
}

/**
  * Given an entry from the output of ftp_rawlist(),
  * attempts to return the permissions of the item.
  *
  * @param string $item Entry from ftp_rawlist()
  * @return string Permissions
  */
function getPermissions($item)
{
    $matches = array();

    eregi("^(.{10}) ", $item, $matches);
    return $matches[1];
}

/**
  * Given a file entry from the output of ftp_rawlist(),
  * attempts to return the size of the file in bytes.
  *
  * @param string $item Entry from ftp_rawlist()
  * @return mixed Size in bytes
  */
function getSize($item)
{
    $matches = array();
    eregi("^.{10}[ ]+[0-9]+ [^ ]+ +[^ ]+ +([0-9]+) ", $item, $matches);

    return $matches[1];
}

/**
  * Given a file entry from the output of ftp_rawlist(),
  * attempts to return the owner of the file
  *
  * @param string $item Entry from ftp_rawlist()
  * @return string Owner
  */
function getOwner($item)
{
    $matches = array();
    eregi("^.{10}[ ]+[0-9]+ ([^ ]+) ", $item, $matches);

    return $matches[1];
}

/**
  * Given a file entry from the output of ftp_rawlist(),
  * attempts to return the group of the file
  *
  * @param string $item Entry from ftp_rawlist()
  * @return string Group name
  */
function getGroup($item)
{
    $matches = array();
    eregi("^.{10}[ ]+[0-9]+ [^ ]+ +([^ ]+) ", $item, $matches);

    return $matches[1];
}

/**
  * Attempts to delete a directory and its contents recursively. (This
  * doesn't work yet.)
  * 
  * @param string $dir Directory to delete
  * @return bool TRUE on success, FALSE on failure.
  */
function recurseDelete($dir)
{
    global $systype, $conn;

    $rawlist=ftp_rawlist($conn, $dir);

    foreach ($rawlist as $item)
    {
        // If it's a file, delete it
        if (isfile($item))
        {
            if (!@ftp_delete($conn, $dir."/".getName($item)))
            {
                return false;
            } // Can't delete
        }

        // If it's a directory, then recurse
        if (isdir($item))
        {
            if (!(recurseDelete($dir."/".getName($item))))
            {
                return false;
            } // Can't delete
        }
    }

    // Now remove the directory itself
    if (!@ftp_rmdir($conn, $dir))
    {
        return false;
    } // Can't remove

    return true;
}
?>
